<?php

declare(strict_types=1);

namespace Drupal\tmgmt_workspaces;

use Drupal\tmgmt\Entity\Job;

/**
 * Entity class override for the tmgmt_job entity.
 */
class WorkspaceAwareJob extends Job {

  /**
   * {@inheritdoc}
   */
  public function getConflictingItemIds() {
    // @todo Show the conflicting items in the Job form as well.
    // @see \Drupal\tmgmt\Form\JobForm::getConflictingItemsMessage()
    return array_unique(array_merge(parent::getConflictingItemIds(), $this->getWorkspaceConflictingItemsIds()));
  }

  /**
   * Returns workspace-conflicting job item IDs.
   *
   * Workspace-conflicting job items are those that have the source entity
   * already tracked in a different workspace.
   *
   * @return int[]
   *   List of conflicting job item IDs.
   */
  public function getWorkspaceConflictingItemsIds(): array {
    // Return early if there's no target workspace configured.
    if ($this->get('target_workspace')->isEmpty()) {
      return [];
    }

    /** @var \Drupal\workspaces\WorkspaceAssociationInterface $workspace_association */
    $workspace_association = \Drupal::service('workspaces.association');
    $target_workspace = $this->get('target_workspace')->target_id;

    $workspace_conflicting_item_ids = [];
    foreach ($this->getItems() as $item) {
      if ($item->getPlugin() !== 'content') {
        continue;
      }

      /** @var \Drupal\Core\Entity\RevisionableInterface $entity */
      $entity = $this->entityTypeManager()->getStorage($item->getItemType())->load($item->getItemId());
      if ($tracking_workspace_ids = $workspace_association->getEntityTrackingWorkspaceIds($entity)) {
        if (!in_array($target_workspace, $tracking_workspace_ids, TRUE)) {
          $workspace_conflicting_item_ids[] = $item->id();
        }
      }
    }

    return $workspace_conflicting_item_ids;
  }

}
