<?php

declare(strict_types=1);

namespace Drupal\tmgmt_workspaces;

use Drupal\tmgmt\Entity\JobItem;
use Drupal\workspaces\Entity\Workspace;

/**
 * Entity class override for the tmgmt_job_item entity.
 */
class WorkspaceAwareJobItem extends JobItem {

  /**
   * {@inheritdoc}
   */
  public function acceptTranslation(): bool {
    /** @var \Drupal\workspaces\WorkspaceInformationInterface $workspace_info */
    $workspace_info = \Drupal::service('workspaces.information');
    /** @var \Drupal\workspaces\WorkspaceManagerInterface $workspace_manager */
    $workspace_manager = \Drupal::service('workspaces.manager');

    // Ensure that we don't try to translate unsupported entity types in a
    // workspace.
    // @todo Optimize the supported check with a static cache for the item type.
    $can_translate_in_workspace = $this->getPlugin() === 'content'
      && $workspace_info->isEntityTypeSupported($this->entityTypeManager()->getDefinition($this->getItemType()));

    $job = $this->getJob();
    if ($can_translate_in_workspace && !$job->get('target_workspace')->isEmpty()) {
      $target_workspace = $job->get('target_workspace')->target_id;

      // If the target workspace doesn't exist (e.g. it was deleted) or if it's
      // closed, we can't accept the translation.
      if (!($workspace = Workspace::load($target_workspace)) || wse_workspace_get_status($workspace) != WSE_STATUS_OPEN) {
        return FALSE;
      }

      return $workspace_manager->executeInWorkspace($target_workspace, fn () => parent::acceptTranslation());
    }

    return parent::acceptTranslation();
  }

}
